/*
    This file is part of FireBible, a bible study tool for Firefox.
    Copyright (C) 2008 Brian Fernandes <infernalproteus@gmail.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

package com.thegoan.firebible;

import java.net.URI;

import org.crosswire.common.progress.JobManager;
import org.crosswire.common.progress.Progress;
import org.crosswire.common.progress.WorkEvent;
import org.crosswire.common.progress.WorkListener;
import org.crosswire.common.util.NetUtil;
import org.crosswire.jsword.book.Book;
import org.crosswire.jsword.index.Index;
import org.crosswire.jsword.index.IndexStatus;
import org.crosswire.jsword.index.lucene.LuceneIndex;
import org.crosswire.jsword.index.lucene.LuceneIndexManager;

public class FBIndexManager extends LuceneIndexManager implements WorkListener {
	
	private Exception exception;
	private int work;
	private String workSection;
	private Book book;
	
	public void scheduleIndexCreation(final Book book) {
		this.book = book;
		book.setIndexStatus(IndexStatus.SCHEDULED);
		JobManager.addWorkListener(this);
		Thread work = new Thread(new Runnable() {
			public void run() {
				IndexStatus finalStatus = IndexStatus.UNDONE;
				try {
					URI storage = getStorageArea(book);
					Index index = new LuceneIndex(book, storage, true);
					// We were successful if the directory exists.
					if (NetUtil.getAsFile(storage).exists()) {
						finalStatus = IndexStatus.DONE;
						INDEXES.put(book, index);
					}
				} catch (Exception e) {
					exception = e;
				} finally {
					book.setIndexStatus(finalStatus);
					JobManager.removeWorkListener(FBIndexManager.this);
				}
			}
		});
		work.start();
	}
	
	public int getWork() {
		return work;		
	}
	
	public String getSectionName() {
		return workSection;
	}	
	
	public Exception getException() {
		return exception;
	}
	
	public void workProgressed(WorkEvent event) {
		Progress progress = event.getJob();
		work = progress.getWork();
		workSection = progress.getSectionName();
	}

	public void workStateChanged(WorkEvent event) {
		//do nothing
	}
	
	public boolean isComplete() {
		IndexStatus idxStatus = book.getIndexStatus();
		return idxStatus == IndexStatus.DONE || idxStatus == IndexStatus.UNDONE;
	}
	
	public Object[] getIndexStatus() {
		return new Object[] { new Boolean(isComplete()), new Integer(work),
				workSection, exception };
	}
	
	public Book getBook() {
	    return book;
	}
	
}
