/*
    This file is part of FireBible, a bible study tool for Firefox.
    Copyright (C) 2008 Brian Fernandes <infernalproteus@gmail.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

package com.thegoan.firebible;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.xml.transform.TransformerException;

import org.crosswire.common.xml.Converter;
import org.crosswire.common.xml.SAXEventProvider;
import org.crosswire.common.xml.TransformingSAXEventProvider;
import org.crosswire.common.xml.TransformingSAXEventProviderConverter;
import org.crosswire.common.xml.XMLUtil;
import org.crosswire.jsword.book.Book;
import org.crosswire.jsword.book.BookData;
import org.crosswire.jsword.book.BookException;
import org.crosswire.jsword.book.BookFilters;
import org.crosswire.jsword.book.BookMetaData;
import org.crosswire.jsword.book.Books;
import org.crosswire.jsword.book.sword.SwordBookPath;
import org.crosswire.jsword.index.IndexStatus;
import org.crosswire.jsword.passage.Key;
import org.crosswire.jsword.passage.NoSuchKeyException;
import org.crosswire.jsword.passage.NoSuchVerseException;
import org.crosswire.jsword.versification.BibleInfo;
import org.xml.sax.SAXException;

public class FBUtils {		
	private URI xslURI;
	private File[] augmentPaths;
    private static FBUtils instance;
    private List bibleList;
	
	public static synchronized FBUtils getInstance() {
	    if (instance == null) {
	        instance = new FBUtils();
	    }
	    return instance;
	}
	
	public Books getBooks(File[] dirs) throws BookException {
	    if (!Arrays.equals(augmentPaths, dirs)) {
	        bibleList = null;
            if (dirs != null) {        
                augmentPaths = dirs;
                SwordBookPath.setAugmentPath(dirs);
            }
            //TODO: Find a way to remove folders too
	    }
        return Books.installed();
    }	
	
	public List getBibles() {
	    if (bibleList != null) return bibleList;
	    Books books = Books.installed();
	    List filteredList = books.getBooks(BookFilters.getOnlyBibles());
	    bibleList = new ArrayList();
	    for (int i = 0; i < filteredList.size(); i++) {
	        Book bible = (Book)filteredList.get(i);
	        bibleList.add(new Object[] {bible.getInitials(), bible});
	    }
	    return bibleList;
	}
	
	public Object[] getBooksInfo() throws NoSuchVerseException {
        List info = new ArrayList();
        for (int i = 1; i <= BibleInfo.booksInBible(); i++) {
            String name = BibleInfo.getBookName(i).toString();
            String shortName = BibleInfo.getShortBookName(i);
            int chapters = BibleInfo.chaptersInBook(i);
            info.add(new Object[] { name, shortName, new Integer(chapters) });
        }
        return info.toArray();
    }
	
	public void setXSLURI(String uri) throws URISyntaxException {
		xslURI = new URI(uri);
	}

	public String getHTMLString(Book bible, String keyString, URLClassLoader loader)
			throws NoSuchKeyException, BookException, TransformerException,
			SAXException {

		Thread.currentThread().setContextClassLoader(loader);

		Key key = null;
		try {
			key = bible.getKey(keyString);
			BookData data = new BookData(bible, key);
			SAXEventProvider osissep = data.getSAXEventProvider();

			Converter styler = getConverter();

			TransformingSAXEventProvider htmlsep = (TransformingSAXEventProvider) styler
					.convert(osissep);

			// You can also pass parameters to the xslt. What you pass depends
			// upon
			// what the xslt can use.
			BookMetaData bmd = bible.getBookMetaData();
			boolean direction = bmd.isLeftToRight();
			htmlsep.setParameter("direction", direction ? "ltr" : "rtl");
			htmlsep.setParameter("pageTitle", keyString);
			htmlsep.setParameter("font", "sans-serif,0,16");

			return XMLUtil.writeToString(htmlsep);
		} catch (Exception e) {
			return e.getMessage();
		}
	}
	
	public String getHTML(Book bible, String keyString,
			URLClassLoader loader) throws NoSuchKeyException, BookException,
			TransformerException, SAXException, UnsupportedEncodingException {
		String html = getHTMLString(bible, keyString, loader);
		return html;
	}	
	
	public boolean isIndexed(Book book) {
		return book.getIndexStatus() == IndexStatus.DONE;
	}	

	public FBIndexManager index(Book book) {
		FBIndexManager indexManager = new FBIndexManager();
		indexManager.scheduleIndexCreation(book);
		return indexManager;
	}
	
	public Converter getConverter() {
		// Converter converter = ConverterFactory.getConverter();
		TransformingSAXEventProviderConverter converter = 
			new TransformingSAXEventProviderConverter(xslURI);
		return converter;
	}

}
